<?php
class pColor {

	/**
	 * @var array
	 */
	private $rgb	= array('R'=>0,'G'=>0,'B'=>0,'Alpha'=>100);
	
	/**
	 * Small helper to determine the order of the colors
	 * @var array
	 */
	private $_order	= array('R','G','B');
	
	
	/**
	 * Constructor
	 * @param string|array $color input as hex "#ff0000" or array('R'=>0,'G'=>0,'B'=>0) 
	 */
	public function __construct($color){
		$this->set($color);
	}
	
	/**
	 * Set the color
	 * @param string|array $color input as hex "#ff0000" or array('R'=>0,'G'=>0,'B'=>0) 
	 * @return pColor this object
	 */
	public function set($color){
		if(is_array($color)){
			// Merge colors
			foreach($this->rgb as $key=>$value){
				if(@$color[$key]){
					$this->rgb[$key]	= $color[$key];
				}
			}
		}else{
			$this->_convertToRGB($color);
		}
		return $this;
	}

	/**
	 * Get the hex value of the color. Example "#ff0000"
	 * @param bool $includeAlpha include the alpha as last hex value
	 */
	public function getHex($includeAlpha=false){
		return $this->_convertToHex($includeAlpha);
	}

	/**
	 * Make a clone of this pColor
	 * @return pColor
	 */
	public function copy(){
		return clone $this;
	}

	/**
	 * Converts HEX format to RGB(A) format
	 * @param string $color
	 * @return bool successfull or not
	 */
	private function _convertToRGB($color){
		$color = str_replace('#', '', $color);
	    if (strlen($color) < 6){ 
	    	return false;
	    }
		
	    $this->rgb	= array('R'=>0,'B'=>0,'B'=>0,'Alpha'=>100);
	    
	    $x=0;
	    foreach($this->_order as $id){
	        $this->rgb[$id] = hexdec(substr($color,(2*$x),2));
	        $x++;
	    }
	    // Check for rgba format
	    if (strlen($color) ==8 ){ 
	        $this->rgb['Alpha'] = floor(100*hexdec(substr($color,6,2))/255);
	    }
	    return true;
	}
	
	/**
	 * Convert from RGB(A) to HEX
	 * @param bool $includeAlpha Include the alpha value?
	 * @return string 
	 */
	private function _convertToHex($includeAlpha=false){
		$color='#';
	    foreach($this->_order as $key){
	    	$val	= $this->rgb[$key];
	    	if($val<16){
	    		$color.='0';
	    	}
	    	$color .= dechex($val);
	    }
	    if($includeAlpha){
	    	$val	= $this->rgb['Alpha'];
	    	if($val<16){
	    		$color.='0';
	    	}
	    	$color .= dechex($val);
	    }
	    return $color;
	}

	/**
	 * Make the color brighter
	 * @param float $factor
	 * @return pColor this object
	 */
	public function brighter($factor=0.4){
		foreach($this->_order as $c){
			$val	= $this->rgb[$c];
			$val	= floor($val+(255-$val)*$factor);
			if($val>255){ $val=255; }
			$this->rgb[$c]	= $val;
		}
		return $this;
	}

	/**
	 * Make the color darker
	 * @param float $factor
	 * @return pColor this object
	 */
	public function darker($factor=0.4){
		foreach($this->_order as $c){
			$val	= $this->rgb[$c];
			$val	= floor($val*(1-$factor));
			if($val<0){ $val=0; }
			$this->rgb[$c]	= $val;
		}
		return $this;
	}

	/**
	 * Get the color
	 * @param string $prefix
	 * @param string $sufix
	 * @param bool $includeAlpha Include the alpha or not
	 */
	public function get($prefix='',$sufix='',$includeAlpha=true){
    	$color=array();
	    $color[$prefix.'R'.$sufix]	= $this->rgb['R'];
	    $color[$prefix.'G'.$sufix]	= $this->rgb['G'];
		$color[$prefix.'B'.$sufix]	= $this->rgb['B'];
		if($includeAlpha){
			$color[$prefix.'Alpha'.$sufix]	= $this->rgb['Alpha'];
		}
	    return $color;
	}
}